<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

defined('MOODLE_INTERNAL') || die();

use assignsubmission_dta\db_utils;
use assignsubmission_dta\dta_backend_utils;
use assignsubmission_dta\view_submission_utils;
use assignsubmission_dta\models\dta_result;
use assignsubmission_dta\models\dta_result_summary;
use assignsubmission_dta\models\dta_recommendation;

/**
 * Library class for DTA submission plugin extending assign submission plugin base class
 *
 * @package assignsubmission_dta
 * @license   http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */
class assign_submission_dta extends assign_submission_plugin {

    /**
     * Broadly used in logic, parametrized for easier change.
     */
    const COMPONENT_NAME = "assignsubmission_dta";
    /**
     * Draft file area for DTA tests to be uploaded by the teacher.
     */
    const ASSIGNSUBMISSION_DTA_DRAFT_FILEAREA_TEST = "tests_draft_dta";
    /**
     * File area for DTA tests to be uploaded by the teacher.
     */
    const ASSIGNSUBMISSION_DTA_FILEAREA_TEST = "tests_dta";
    /**
     * File area for DTA submission assignment.
     */
    const ASSIGNSUBMISSION_DTA_FILEAREA_SUBMISSION = "submissions_dta";

    /**
     * Get plugin name
     * @return string
     */
    public function get_name(): string {
        return get_string("pluginname", self::COMPONENT_NAME);
    }

    /**
     * Get default settings for assignment submission settings
     *
     * @param MoodleQuickForm $mform form to add elements to
     * @return void
     */
    public function get_settings(MoodleQuickForm $mform): void {
        // Add draft filemanager to form.
        $mform->addElement(
            "filemanager",
            self::ASSIGNSUBMISSION_DTA_DRAFT_FILEAREA_TEST,
            get_string("submission_settings_label", self::COMPONENT_NAME),
            null,
            $this->get_file_options(true)
        );

        // Add help button to added filemanager.
        $mform->addHelpButton(
            // Form-unique element id to which to add button.
            self::ASSIGNSUBMISSION_DTA_DRAFT_FILEAREA_TEST,
            // Key.
            "submission_settings_label",
            // Language file to use.
            self::COMPONENT_NAME
        );

        // Only show filemanager if plugin is enabled.
        $mform->hideIf(
            // Form-unique element id to hide.
            self::ASSIGNSUBMISSION_DTA_DRAFT_FILEAREA_TEST,
            // Condition to check.
            self::COMPONENT_NAME . '_enabled',
            // State to match for hiding.
            'notchecked'
        );
    }

    /**
     * Allows the plugin to update the default values passed into
     * the settings form (needed to set up draft areas for editor
     * and filemanager elements)
     * @param array $defaultvalues
     */
    public function data_preprocessing(&$defaultvalues): void {
        // Get id of draft area for file manager creation.
        $draftitemid = file_get_submitted_draft_itemid(self::ASSIGNSUBMISSION_DTA_DRAFT_FILEAREA_TEST);

        // Prepare draft area with created draft filearea.
        file_prepare_draft_area(
            $draftitemid,
            $this->assignment->get_context()->id,
            self::COMPONENT_NAME,
            self::ASSIGNSUBMISSION_DTA_FILEAREA_TEST,
            0,
            ['subdirs' => 0]
        );

        $defaultvalues[self::ASSIGNSUBMISSION_DTA_DRAFT_FILEAREA_TEST] = $draftitemid;
    }

    /**
     * Save settings of assignment submission settings
     *
     * @param stdClass $data
     * @return bool
     */
    public function save_settings(stdClass $data): bool {

        // If the assignment has no filemanager for our plugin just leave.
        $draftfilemanagerid = self::ASSIGNSUBMISSION_DTA_DRAFT_FILEAREA_TEST;
        if (!isset($data->$draftfilemanagerid)) {
            return true;
        }

        // Store files from draft filearea to final one.
        file_save_draft_area_files(
            // Form-unique element id of draft filemanager from the edit.
            $data->$draftfilemanagerid,
            // Id of the assignment in edit.
            $this->assignment->get_context()->id,
            self::COMPONENT_NAME,
            self::ASSIGNSUBMISSION_DTA_FILEAREA_TEST,
            0
        );

        // Get files from proper filearea.
        $fs = get_file_storage();
        $files = $fs->get_area_files(
            // Id of the current assignment.
            $this->assignment->get_context()->id,
            self::COMPONENT_NAME,
            self::ASSIGNSUBMISSION_DTA_FILEAREA_TEST,
            0,
            'id',
            false
        );

        // Check if a file was uploaded.
        if (empty($files)) {
            \core\notification::error(get_string("no_testfile_warning", self::COMPONENT_NAME));
            return true;
        }

        // Get the file.
        $file = reset($files);

        // Send file to backend.
        return dta_backend_utils::send_testconfig_to_backend($this->assignment, $file);
    }

    /**
     * Add elements to submission form
     *
     * @param mixed $submissionorgrade stdClass|null submission or grade to show in the form
     * @param MoodleQuickForm $mform form for adding elements
     * @param stdClass $data data for filling the elements
     * @param int $userid current user
     * @return bool form elements added
     */
    public function get_form_elements_for_user($submissionorgrade, MoodleQuickForm $mform, stdClass $data, $userid): bool {
        // Prepare submission filearea.
        $data = file_prepare_standard_filemanager(
            $data,
            'tasks',
            $this->get_file_options(false),
            $this->assignment->get_context(),
            self::COMPONENT_NAME,
            self::ASSIGNSUBMISSION_DTA_FILEAREA_SUBMISSION,
            $submissionorgrade ? $submissionorgrade->id : 0
        );

        // Add filemanager to form.
        $mform->addElement(
            'filemanager',
            // Form-unique identifier.
            'tasks_filemanager',
            // Label to show next to the filemanager.
            get_string("submission_label", self::COMPONENT_NAME),
            // Attributes.
            null,
            // Options.
            $this->get_file_options(false)
        );

        // Add help button.
        $mform->addHelpButton(
            // Related form item.
            "tasks_filemanager",
            // Key.
            "submission_label",
            // Language file.
            self::COMPONENT_NAME
        );

        return true;
    }

    /**
     * Determines if a submission file area contains any files.
     * @param stdClass $submission submission to check
     * @return bool true if file count is zero
     */
    public function is_empty(stdClass $submission): bool {
        return $this->count_files($submission->id, self::ASSIGNSUBMISSION_DTA_FILEAREA_SUBMISSION) == 0;
    }

    /**
     * Counts the number of files in a filearea
     *
     * @param int $submissionid submission id to check
     * @param string $areaid filearea id to count
     * @return int number of files submitted in the filearea
     */
    private function count_files(int $submissionid, $areaid) {
        $fs = get_file_storage();
        $files = $fs->get_area_files($this->assignment->get_context()->id,
            self::COMPONENT_NAME,
            $areaid,
            $submissionid,
            'id',
            false);

        return count($files);
    }

    /**
     * Save data to the database
     *
     * @param stdClass $submission
     * @param stdClass $data
     * @return bool
     */
    public function save(stdClass $submission, stdClass $data) {
        $data = file_postupdate_standard_filemanager(
            $data,
            'tasks',
            $this->get_file_options(false),
            $this->assignment->get_context(),
            self::COMPONENT_NAME,
            self::ASSIGNSUBMISSION_DTA_FILEAREA_SUBMISSION,
            $submission->id
        );

        // If submission is empty, leave directly.
        if ($this->is_empty($submission)) {
            return true;
        }

        // Get submitted files.
        $fs = get_file_storage();
        $files = $fs->get_area_files(
            // Id of current assignment.
            $this->assignment->get_context()->id,
            self::COMPONENT_NAME,
            self::ASSIGNSUBMISSION_DTA_FILEAREA_SUBMISSION,
            $submission->id,
            'id',
            false
        );

        // Check if a file is uploaded.
        if (empty($files)) {
            \core\notification::error(get_string("no_submissionfile_warning", self::COMPONENT_NAME));
            return true;
        }

        // Get the file.
        $file = reset($files);

        // Send file to backend.
        $response = \assignsubmission_dta\dta_backend_utils::send_submission_to_backend($this->assignment, $submission->id, $file);

        // With a null response, return an error.
        if (is_null($response)) {
            return false;
        }

        // Convert received JSON to valid class instances.
        $resultsummary = dta_result_summary::decode_json($response);

        // Decode recommendations from response.
        $recommendations = dta_recommendation::decode_json_recommendations($response);

        error_log(print_r($recommendations, true));

        // Persist new results to database.
        db_utils::store_result_summary_to_database($this->assignment->get_instance()->id, $submission->id, $resultsummary);

        // Store the array of recommendations in the database.
        db_utils::store_recommendations_to_database($this->assignment->get_instance()->id, $submission->id, $recommendations);

        return true;
    }

    /**
     * Display a short summary of the test results of the submission
     * This is displayed as default view, with the option to expand
     * to the full detailed results.
     *
     * @param stdClass $submission to show
     * @param bool $showviewlink configuration variable to show expand option
     * @return string summary results html
     */
    public function view_summary(stdClass $submission, & $showviewlink) {
        $showviewlink = true;

        return view_submission_utils::generate_summary_html(
            $this->assignment->get_instance()->id,
            $submission->id
        );
    }

    /**
     * Display detailed results
     *
     * @param stdClass $submission the submission the results are shown for.
     * @return string detailed results html
     */
    public function view(stdClass $submission) {
        return view_submission_utils::generate_detail_html(
            $this->assignment->get_instance()->id,
            $submission->id
        );
    }

    /**
     * Generate array of allowed file types to upload.
     *
     * @param bool $settings switch to define if list for assignment settings
     *      or active submission should be returned
     *
     * @return array
     */
    private function get_file_options(bool $settings): array {
        $fileoptions = [
                'subdirs' => 0,
                "maxfiles" => 1,
                'accepted_types' => ($settings
                    ? [".txt"]
                    : [
                        ".txt",
                        ".zip",
                    ]),
                'return_types' => FILE_INTERNAL,
            ];
        return $fileoptions;
    }

    /**
     * Get file areas returns a list of areas this plugin stores files
     * @return array - An array of fileareas (keys) and descriptions (values)
     */
    public function get_file_areas() {
        return [
            self::ASSIGNSUBMISSION_DTA_FILEAREA_SUBMISSION => get_string("dta_submissions_fa", self::COMPONENT_NAME),
            self::ASSIGNSUBMISSION_DTA_FILEAREA_TEST => get_string("dta_tests_fa", self::COMPONENT_NAME),
        ];
    }

    /**
     * Produce a list of files suitable for export that represent this feedback or submission
     *
     * @param stdClass $submission The submission
     * @param stdClass $user The user record - unused
     * @return array - return an array of files indexed by filename
     */
    public function get_files(stdClass $submission, stdClass $user) {
        $result = [];
        $fs = get_file_storage();
        $files = $fs->get_area_files($this->assignment->get_context()->id,
            self::COMPONENT_NAME,
            self::ASSIGNSUBMISSION_DTA_FILEAREA_SUBMISSION,
            $submission->id,
            'timemodified',
            false);

        foreach ($files as $file) {
            // Do we return the full folder path or just the file name?
            if (isset($submission->exportfullpath) && $submission->exportfullpath == false) {
                $result[$file->get_filename()] = $file;
            } else {
                $result[$file->get_filepath().$file->get_filename()] = $file;
            }
        }
        return $result;
    }

    /**
     * The plugin is being uninstalled - cleanup
     *
     * @return bool
     */
    public function delete_instance() {
        db_utils::uninstall_plugin_cleanup();

        return true;
    }
}
